use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.19

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/Date/Utility.pm',
    't/00-check-deps.t',
    't/00-compile.t',
    't/00-load.t',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/attributes.t',
    't/constructor.t',
    't/mocktime.t',
    't/object_methods.t',
    't/perlcritic.t',
    't/rc/.perlcriticrc',
    't/rc/.perltidyrc',
    't/static_methods.t',
    't/today.t'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
